/*
 * Copyright 2008 Sony Corporation of America
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this Library and associated documentation files (the
 * "Library"), to deal in the Library without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Library, and to
 * permit persons to whom the Library is furnished to do so, subject to
 * the following conditions:
 *
 *  The above copyright notice and this permission notice shall be
 *  included in all copies or substantial portions of the Library.
 *
 *  If you modify the Library, you may copy and distribute your modified
 *  version of the Library in object code or as an executable provided
 *  that you also do one of the following:
 *
 *   Accompany the modified version of the Library with the complete
 *   corresponding machine-readable source code for the modified version
 *   of the Library; or,
 *
 *   Accompany the modified version of the Library with a written offer
 *   for a complete machine-readable copy of the corresponding source
 *   code of the modified version of the Library.
 *
 *
 * THE LIBRARY IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
 * CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 * LIBRARY OR THE USE OR OTHER DEALINGS IN THE LIBRARY.
 */

#include "config.h"

#include "mars/module.h"

#include "kernel_internal_types.h"
#include "workload_internal_types.h"

static struct mars_kernel_syscalls *kernel_syscalls;

void mars_module_entry(struct mars_kernel_syscalls *syscalls)
{
	kernel_syscalls = syscalls;

	asm volatile (
		/* switch to module stack */
		"ila	$sp, __stack;"

		/* set module link register to NULL */
		"il	$lr, 0;"

		/* initialize back chain to NULL */
		"stqd	$lr, 0($sp);"

		/* initialize module stack frame */
		"stqd	$sp, -32($sp);"
		"ai	$sp, $sp, -32;"

		/* save link register in link register save area */
		"stqd	$lr, 16($sp);"

		/* call _init function */
		"brsl	$lr, _init;"
	);

	mars_module_main();

	(*kernel_syscalls->workload_exit)(MARS_WORKLOAD_STATE_FINISHED);
}

uint32_t mars_module_get_ticks(void)
{
	return (*kernel_syscalls->get_ticks)();
}

uint64_t mars_module_get_mars_context_ea(void)
{
	return (*kernel_syscalls->get_mars_context_ea)();
}

uint16_t mars_module_get_kernel_id(void)
{
	return (*kernel_syscalls->get_kernel_id)();
}

uint16_t mars_module_get_workload_id(void)
{
	return (*kernel_syscalls->get_workload_id)();
}

struct mars_workload_context *mars_module_get_workload(void)
{
	return (*kernel_syscalls->get_workload)();
}

struct mars_workload_context *mars_module_get_workload_by_id(uint16_t id)
{
	return (*kernel_syscalls->get_workload_by_id)(id);
}

int mars_module_workload_query(uint16_t id, int query)
{
	return (*kernel_syscalls->workload_query)(id, query);
}

int mars_module_workload_wait_set(uint16_t id)
{
	return (*kernel_syscalls->workload_wait_set)(id);
}

int mars_module_workload_wait_reset(void)
{
	return (*kernel_syscalls->workload_wait_reset)();
}

int mars_module_workload_signal_set(uint16_t id)
{
	return (*kernel_syscalls->workload_signal_set)(id);
}

int mars_module_workload_signal_reset(void)
{
	return (*kernel_syscalls->workload_signal_reset)();
}

int mars_module_workload_schedule_begin(uint16_t id, uint8_t priority,
					struct mars_workload_context **workload)
{
	return (*kernel_syscalls->workload_schedule_begin)(id,
							   priority,
							   workload);
}

int mars_module_workload_schedule_end(uint16_t id)
{
	return (*kernel_syscalls->workload_schedule_end)(id);
}

int mars_module_workload_schedule_cancel(uint16_t id)
{
	return (*kernel_syscalls->workload_schedule_cancel)(id);
}

void mars_module_workload_wait(void)
{
	(*kernel_syscalls->workload_exit)(MARS_WORKLOAD_STATE_WAITING);
}

void mars_module_workload_yield(void)
{
	(*kernel_syscalls->workload_exit)(MARS_WORKLOAD_STATE_READY);
}

void mars_module_workload_finish(void)
{
	(*kernel_syscalls->workload_exit)(MARS_WORKLOAD_STATE_FINISHED);
}

int mars_module_host_signal_send(uint64_t watch_point_ea)
{
	return (*kernel_syscalls->host_signal_send)(watch_point_ea);
}

int mars_module_mutex_lock_get(uint64_t mutex_ea, struct mars_mutex *mutex)
{
	return (*kernel_syscalls->mutex_lock_get)(mutex_ea, mutex);
}

int mars_module_mutex_unlock_put(uint64_t mutex_ea, struct mars_mutex *mutex)
{
	return (*kernel_syscalls->mutex_unlock_put)(mutex_ea, mutex);
}

int mars_module_dma_get(void *ls, uint64_t ea, uint32_t size, uint32_t tag)
{
	return (*kernel_syscalls->dma_get)(ls, ea, size, tag);
}

int mars_module_dma_put(const void *ls, uint64_t ea, uint32_t size,
			uint32_t tag)
{
	return (*kernel_syscalls->dma_put)(ls, ea, size, tag);
}

int mars_module_dma_wait(uint32_t tag)
{
	return (*kernel_syscalls->dma_wait)(tag);
}
